/*
    Copyright (C) 1998, 1999  Michael Ravits

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "include/newplayer.h"
#include "include/player_newplayer.h"
#include "include/screen.h"
#include "include/stuff.h"
#include "include/playlist.h"


screen::screen(stuff &astuff, class playlist &plist, class player &pl)
{
	stuff	some_stuff;
	
	some_stuff.set_app_name(program_name);

	if(!initscr())
	{
		some_stuff.print_error("init_scr() failed.");
		exit(-1);
	}
	
	if(!(main_window = newwin(LINES - 2, COLS - 2, 1 , 1)))
	{
		some_stuff.print_error("newwin() failed.");
		exit(-1);
	}

	if(!(input_window = newwin(3, COLS - 2, LINES / 2 - 2, 1)))
	{
		some_stuff.print_error("newwin() failed.");
	}
	
	if(!(help_window = newwin(LINES - 1, COLS / 2, 1, COLS / 4)))
	{
		some_stuff.print_error("newwin() failed.");
	}
	
	start_color();
	if(has_colors())
	{
		init_pair(1, COLOR_WHITE, COLOR_BLACK);
		init_pair(2, COLOR_BLACK, COLOR_GREEN);
		init_pair(3, COLOR_BLUE , COLOR_BLACK);
		init_pair(4, COLOR_BLACK, COLOR_BLACK);
		init_pair(5, COLOR_WHITE, COLOR_GREEN);
		init_pair(6, COLOR_GREEN, COLOR_BLACK);
	}
	else
	{
		init_pair(1, COLOR_WHITE, COLOR_BLACK);
		init_pair(2, COLOR_BLACK, COLOR_WHITE);
		init_pair(3, COLOR_WHITE, COLOR_BLACK);
		init_pair(4, COLOR_BLACK, COLOR_BLACK);
		init_pair(5, COLOR_WHITE, COLOR_BLACK);
		init_pair(6, COLOR_WHITE, COLOR_BLACK);
	}
			
        timeout(0);
        cbreak();
        keypad(stdscr, TRUE);
        curs_set(FALSE);
	
	screen::cursor = 0;
	screen::idx    = 0;
	song_timecode = song_duration = song_info = NULL;
	range_offset = NULL;
	balance = 50; volume = 80; master_volume = 80;
	
	//screen::draw_screen(astuff, plist, pl);
}

screen::~screen(void)
{
	curs_set(TRUE);
	endwin();
}

void screen::draw_screen(stuff &bstuff, playlist &plist, player &pl)
{
	int	i, in_plist;
	char	*current_file;
	char	volume_str[20];
	
	screen::win_lines = LINES - 2;
	wattrset(stdscr, A_BOLD|COLOR_PAIR(1));
	werase(stdscr);
	werase(main_window);
	box(stdscr, 0, 0);
	if(pl.is_playing() || pl.is_paused())
	{
		mvwaddstr(stdscr, LINES - 1, (COLS / 20), range_offset);
		mvwaddstr(stdscr, LINES - 1, (COLS / 20) * 4, song_timecode);
		mvwaddstr(stdscr, LINES - 1, (COLS / 20) * 7, song_duration);
		mvwaddstr(stdscr, LINES - 1, (COLS / 20) * 10, song_info);
	}
	if(pl.is_paused())
	{
		mvwaddstr(stdscr, 0, COLS - 8, "PAUSED");
	}
	if(plist.rplay_method() == RANDOM)
		mvwaddstr(stdscr, 0, (COLS / 20), "[Random play]");
	else
		mvwaddstr(stdscr, 0, (COLS / 20), "[Alphabetical play]");
	sprintf(volume_str, "[Volume - %d]", master_volume);
	mvwaddstr(stdscr, 0, (COLS / 20) * 6, volume_str);
	mvwaddstr(stdscr, 0, (COLS / 20) * 10, "[Press H/h for help]");
	
	for(i = 0; (i < bstuff.rfiles_num()) && (i < win_lines); i++)
	{
		current_file = bstuff.rfile_name(idx + i);
		in_plist     = plist.in_playlist(current_file);
			
		if((i == cursor)&&(in_plist > -1)) 
			wattrset(main_window, A_BOLD|COLOR_PAIR(5));
		else
		if(i == cursor)
			wattrset(main_window, COLOR_PAIR(2));
		else
		if(in_plist > -1)
			wattrset(main_window, COLOR_PAIR(3));
		else
			wattrset(main_window, A_BOLD|COLOR_PAIR(4));
		if((signed)strlen(current_file) > COLS)
			mvwaddnstr(main_window, i, 0, current_file, COLS - 2);
		else
			mvwaddstr(main_window, i, 0, current_file);
	}
	wnoutrefresh(stdscr);
	wnoutrefresh(main_window);
	if(plist.is_input_window_active())
		wnoutrefresh(input_window);
	if(bstuff.is_help_window_active())
		wnoutrefresh(help_window);
	doupdate();
}

void screen::set_params(int idx, int cursor)
{
	screen::cursor = cursor;
	screen::idx    = idx;
}

void screen::clr_refresh(class stuff *st, class playlist &plist, class player &pl)
{
	werase(stdscr);
	werase(curscr);
	werase(main_window);
	draw_screen(*st, plist, pl);
}

int screen::rcursor(void)
{
	return cursor;
}

int screen::ridx(void)
{
	return idx;
}

int screen::rwin_lines(void)
{
	return win_lines;
}

void screen::key_up(class stuff *st, class playlist &plist, class player &pl)
{
	if(cursor != 0)
		cursor--;
	else
		if(idx != 0)
			idx--;
	draw_screen(*st, plist, pl);
}

void screen::key_down_without_draw(class stuff *st, class playlist &plist, class player &pl)
{
	if((cursor != st->rfiles_num() - 1) && (cursor != win_lines - 1))
		cursor++;
	else
		if((idx != st->rfiles_num() - win_lines) && (cursor != st->rfiles_num() -1))
			idx++;
}

void screen::key_down(class stuff *st, class playlist &plist, class player &pl)
{
	if((cursor != st->rfiles_num() - 1) && (cursor != win_lines - 1))
		cursor++;
	else
		if((idx != st->rfiles_num() - win_lines) && (cursor != st->rfiles_num() -1))
			idx++;
	draw_screen(*st, plist, pl);
}

void screen::key_npage(class stuff *st, class playlist &plist, class player &pl)
{
	int	fnum, i;
	
	fnum   = st->rfiles_num();
	
	for(i = 0; i < win_lines; i++)
	{
		if((cursor != win_lines - 1) && (cursor != fnum - 1))
			cursor++;
		else
			if((idx != fnum - win_lines) && (cursor != fnum - 1))
				idx++;
	}
	
	draw_screen(*st, plist, pl);
}

void screen::key_ppage(class stuff *st, class playlist &plist, class player &pl)
{
	int	i;
	
	for(i = 0; i < win_lines; i++)
	{
		if(cursor != 0)
			cursor--;
		else
			if(idx != 0)
				idx--;
	}
	draw_screen(*st, plist, pl);
}

void screen::key_end(class stuff *st, class playlist &plist, class player &pl)
{
	
	if(st->rfiles_num() >= win_lines)
	{
		cursor = win_lines - 1;
		idx    = st->rfiles_num() - win_lines;
	}
	else
	{
		cursor = st->rfiles_num() - 1;
		idx    = 0;
	}
	draw_screen(*st, plist, pl);
}

void screen::key_home(class stuff *st, class playlist &plist, class player &pl)
{
	cursor = idx = 0;
	key_down_without_draw( st, plist, pl );
	draw_screen(*st, plist, pl);
}

void screen::set_song_range_and_offset(int new_range, int new_offset)
{
	char	ro[40];
	
	song_range = new_range;
	song_offset = new_offset;

	sprintf(ro, "[%d/%d]", song_offset, song_range);

	if(range_offset)
		delete range_offset;

	range_offset = new char[strlen(ro) + 1];
	strcpy(range_offset, ro);
}

void screen::set_song_timecode(char *new_timecode)
{
	if(song_timecode)
		delete song_timecode;
	song_timecode = new char[strlen(new_timecode) + 1];
	strcpy(song_timecode, new_timecode);
}
void screen::set_song_duration(char *new_duration)
{
	if(song_duration)
		delete song_duration;
	song_duration = new char[strlen(new_duration) + 1];
	strcpy(song_duration, new_duration);
}

void screen::set_song_info(char *new_info)
{
	if(song_info)
		delete song_info;
	song_info = new char[strlen(new_info) + 1];
	strcpy(song_info, new_info);
}

void screen::set_balance(int new_balance)
{
	balance = new_balance;
}

void screen::set_volume(int new_volume)
{
	volume = new_volume;
}

void screen::set_master_volume(int new_master_volume)
{
	master_volume = new_master_volume;
}

int screen::rsong_offset(void)
{
	return song_offset;
}

int screen::rsong_range(void)
{
	return song_range;
}

int screen::rbalance(void)
{
	return balance;
}

int screen::rvolume(void)
{
	return volume;
}

int screen::rmaster_volume(void)
{
	return master_volume;
}

WINDOW *screen::rinput_window(void)
{
	return input_window;
}

void screen::set_mvb(int m, int v, int b)
{
	set_master_volume(m);
	set_volume(v);
	set_balance(b);
}

WINDOW *screen::rhelp_window(void)
{
	return help_window;
}
